/*.....................................................................................*/
/* lines.c - program to test line drawing functinos.
/*.....................................................................................*/

#include "huc.h"

/*......................................................................................*/
/* the (more or less) standard includes.                                                */
/*......................................................................................*/

#include "Font/Font.c"
#include "Lib/JoyPad.c"
#include "Lib/tap.c"

#include "Sound/psglib.c"
#include "Sound/sndDefs.h"

/*......................................................................................*/
/*
#include "Retro.h"
*/
/*......................................................................................*/
/* the music / effects functions.                                                       */
/*......................................................................................*/

#include "snginit.c"

/*......................................................................................*/

#incspr(sprites, "graphics/bikes.pcx",0,0,8,2)
#incspr(cursor, "graphics/crs.pcx")
#incpal(curPal, "graphics/crs.pcx")
#incpal(racer1Pal, "graphics/bikes.PCX"  ) 
#incpal(racer2Pal, "graphics/palette2.pcx" )
#incpal(racer3Pal, "graphics/palette3.pcx" )
#incpal(racer4Pal, "graphics/palette4.pcx" )
#incpal(racer5Pal, "graphics/palette5.pcx" )

#incchr(linesFont, "graphics/cfont.pcx")
#incpal(fontPal, "graphics/CFONT.PCX" ) 

#define RacerAddress       0x0C00
#define ExplodeAddress     0x0F80

#define FontPalette        	0
#define RacerPalette        16
#define Racer2Palette       17
#define Racer3Palette       18
#define Racer4Palette       19
#define Racer5Palette       20

#define  TRAIL_LENGTH   10

#define  MOVE_UP        0
#define  MOVE_RIGHT     1
#define  MOVE_DOWN      2
#define  MOVE_LEFT      3

int  frame;
int  Wins[5];
int  isDead[5];
int  allDead;

const  int  DeltaX[4]= {  0, 1, 0, -1 };
const  int  DeltaY[4]= { -1, 0, 1,  0 };

/*------------------------------------------------------------------------------------*/
/* subroutines and such.                                                              */
/*......................................................................................*/

#include "title1.c"
#include "racers.c"
#include "que.c"
#include "getDir.c"
#include "border.c"
#include "round.c"
/*......................................................................................*/


main()
{
  int  OverallWinner;

  frame = 0;  

  /*..............................................................................*/
  /* get flags for who is playing, set game up.                                   */
  /*..............................................................................*/
  
  while( 1 )
  {
	title();  
	initGame();
	initRacers();

	OverallWinner = playGame();

    showOverallWinner( OverallWinner );	
  }
}
/*.................................................................................*/
/* initGame() - initialize whole game.                                             */
/*.................................................................................*/

initGame()
{
  /*......................................................................*/
  /* re-set screen, clear everything out.                                 */
  /*......................................................................*/
  
  reset_satb();
  
  /*......................................................................*/
  /* set up font for messages.                                            */
  /*......................................................................*/
  
  load_palette( FontPalette, fontPal, 1 ); 
  load_font(    linesFont, 64, FontBaseAddress );
  set_font_pal(  FontPalette     );

}
/*.................................................................................*/
/* playGame() - play a game (best of 4 rounds)                                     */
/*.................................................................................*/

playGame()
{
  int  i;
  int  TheWinner;  
  int  GameOver;
  
  
  /*..............................................................................*/
  /* everyone starts with 0 wins.                                                 */
  /*..............................................................................*/

  for( i = 0; i < 5; i++  )
  {
     Wins[i] = 0;
  }

  /*..............................................................................*/
  /* play a round of the game, see who won.                                       */
  /*..............................................................................*/

  GameOver = 0;
  while( !GameOver )
  {
	TheWinner = playRound();
	hideRacers();
	
	if( TheWinner >= 0 )
	{
	   Wins[TheWinner]++;
	   GameOver = (Wins[TheWinner] >= 4);
	}
    displayWins();
  }
  return( TheWinner );
}
/*.................................................................................*/
/* displayWins() - display current wins for each player.                           */
/*.................................................................................*/

displayWins()
{
   int  i;
   
   cls();
   for( i = 0; i < 5; i ++ )
   {
      if( flags[i] )
	  {
		  put_string( "PLAYER ",     7, 10+i );
		  put_number( i+1, 3,       14, 10+i );
		  put_number( Wins[i], 3,   19, 10+i );
	  }
   }
   for( i = 0; i < 180; i ++ )
   {
      vsync(0);
   }
}

hideRacers()
{
   int i;
   
   for( i = 0; i < 5; i ++ )
   {
      spr_hide( i + 1 );
   }
   satb_update();
}

showOverallWinner( OverallWinner )
int                OverallWinner;
{
    int  i;
	
	
	cls();
	put_string( "PLAYER ",             8, 9 );
	put_number( OverallWinner + 1, 2, 14, 9 );
	put_string( "WINS",               17, 9 );
	
	put_string( "PRESS RUN TO",        8, 11 );
    put_string( " PLAY AGAIN ",		   8, 12 );
	
	while( 1 )
	{
	   vsync( 0 );
	   
	   for( i = 0; i < 5; i ++ )
	   {
	      if( joy( i ) & JOY_STRT )  return;
	   }
       if( joySelectReleased( 0 ) ) reboot();
	}
}
/*......................................................................................*/
/* reboot() - reboot the system.                                                        */
/*......................................................................................*/

reboot()
{
	#asm
	jmp		_reset
	#endasm
}
