/***************************************************************************/
/* Ball.c - functions for managing ball.                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 2011 By Michael E. Ward and Andrew 'Arkhan' Darovich		*/
/*				All Rights Reserved.               										*/
/*.........................................................................*/


/***************************************************************************/
/* ball_SetupSprite() - functions for initializing ball sprite.            */
/*-------------------------------------------------------------------------*/

ball_SetupSprite()
{
    int  i;
	
	for( i = 0; i < BallTrails; i ++ )
	{
		spr_set(     BallSpriteNo + i );
		spr_x(       Ball_X );
		spr_y(       Ball_Y );
		spr_pattern( BallAddress );          /* All the trails use the same graphic.. */
		spr_pal(     BallPaletteNo + 16  	);          /* uses sprite palette area */
		spr_pri( 1 );
		spr_show(    BallSpriteNo + i    );
	}
}
/***************************************************************************/
/* ball_Setup() - initialize ball, including the sprite.                   */
/*-------------------------------------------------------------------------*/

ball_Setup()
{
	ball_Reset();
	ball_SetupSprite();
	
	BallState = 0;
	trailFrame = 0;
}

/***************************************************************************/
/* ball_Display() - display ball.                                          */
/*-------------------------------------------------------------------------*/

ball_Display()
{
	spr_set(     BallSpriteNo );
	spr_x(       Ball_X);
	spr_y(       Ball_Y);
	spr_show(    BallSpriteNo     );
}

/***************************************************************************/
/* ball_Move() - update ball information.                                  */
/*-------------------------------------------------------------------------*/

ball_Move()
{
    ball_UpdateTrails();
	
	/*............................................................*/
	/* update ball position.                                      */
	/*............................................................*/

    ballAddDeltaX();
	ballAddDeltaY();
	
	/*............................................................*/
	/* did we hit a wall ?                                        */
	/*............................................................*/
	
	if( Ball_top <= 24 )
	{
	   psgPlay(2);
	   ballFlipDeltaY();
	}
	if( Ball_bot >= 216 )
	{
	   psgPlay(1);
	   ballFlipDeltaY();
	}
	
	/*............................................................*/
	/* did we hit the back wall of court?                         */
	/*............................................................*/
	/* note: this stops the ball before it runs into the actual   */
	/* border of the board. That's why it uses 8 and 248          */
	/*............................................................*/
	
	if( Ball_left <= 8 )
	{
	   psgPlay(1);
	   return( -1 );
	}
	if( Ball_right >= 248 )
	{
	   psgPlay(2);
	   return( 1 );
	}
	/*............................................................*/
	/* did we hit player paddle ?                                 */
	/*............................................................*/
	
	if( Ball_Dx < 0 )
	{
		/*............................................................*/
		/* Ball is moving to left. Check if it's in Player 1's paddle.*/
		/*............................................................*/

	   if( Ball_left <= Paddle_1_right )
	   {
		  /*............................................................*/
		  /* ball is not past paddle. If outside box, we're done.       */
		  /* (and no one has scored: that's what the 0 means).          */
		  /*............................................................*/

  	      if( Ball_left < Paddle_1_left ) return( 0 );
	      if( Ball_bot  < Paddle_1_top  ) return( 0 );
		  if( Ball_top  > Paddle_1_bot  ) return( 0 );

		  /*...................................................*/
		  /* where did ball hit paddle, vertically?            */
		  /*...................................................*/
          /* ResetDeltaX returns positive numbers, so using it */
		  /* automatically flips Delta X.                      */
		  /*...................................................*/

		  psgPlay(1);

		  /*...................................................*/
		  /* decide which pixel the ball hit the paddle at.    */
		  /*...................................................*/
		  
		  Ball_DeltaIX = ( Ball_Y - Paddle_1_Y );
/*		  put_number( Ball_DeltaIX, 4, 20, 1 ); */
		  
		  /*......................................................*/
		  /* get new ball Y motion, based on where we hit paddle. */
		  /*......................................................*/
		  /* if ball is moving down, it keeps moving down.        */
		  /* if ball is moving up, we need negative of value      */
		  /* from table. Note that this would be faster if the    */
		  /* table hadn't changed to be all positives.            */
		  /*......................................................*/
		  /* also, arkhan changed the sprite locations to be the  */
		  /* upper-left corners. So our index is no longer +/-.   */
		  /*......................................................*/
		  /*  
		  if( Ball_Dy == 0 )
		  {
			ballResetDeltaY();
			if( Ball_DeltaIX < 12 )
			{
			    ballFlipDeltaY();
			}
		  }
		  */
		  if( Ball_Dy >= 0 )
		  {
			ballResetDeltaY();
		  }
		  else
		  {
			ballResetDeltaY();
			ballFlipDeltaY();
		  }

		  /*.....................................................*/
		  /* Since we have hit the paddle, we need to flip the   */
		  /* X motion, and speed the ball up.                    */
		  /* note that speedup() does this every return :)       */
		  /*.....................................................*/
		  /* also note the order: BallDx is < 0, so we flip it   */
		  /* before we speed the ball up.                        */
		  /*.....................................................*/

		  ballFlipDeltaX(); 
		  speedup();

		  /*.....................................................*/
		  /* limit the speed of the ball.                        */
		  /*.....................................................*/
		  
		  if( Ball_Dx >= 8 )
		  {
				Ball_Fdx = 0;
		  }
			
	   }
	   return( 0 );
	}

	if( Ball_Dx > 0 )
	{
		/*.............................................................*/
		/* Ball is moving to right. Check if it's in Player 2's paddle.*/
		/*.............................................................*/
		
	   if( Ball_right >= Paddle_2_left )    
	   {
		  /*............................................................*/
		  /* ball is not past paddle. If outside box, we're done.       */
		  /* (and no one has scored: that's what the 0 means).          */
		  /*............................................................*/

	      if( Ball_right > Paddle_2_right ) return( 0 );
	      if( Ball_bot   < Paddle_2_top   ) return( 0 );
		  if( Ball_top   > Paddle_2_bot   ) return( 0 );

		  /*...................................................*/
		  /* where did ball hit paddle, vertically?            */
		  /*...................................................*/
          /* ResetDeltaX returns positive numbers, so using it */
		  /* means we need to flip Delta X.                    */
		  /*...................................................*/

		  psgPlay(2);

		  /*...................................................*/
		  /* decide which pixel the ball hit the paddle at.    */
		  /*...................................................*/
		  

		  Ball_DeltaIX = ( Ball_Y - Paddle_2_Y );
/*		  put_number( Ball_DeltaIX, 4, 20, 1 ); */

		  /*......................................................*/
		  /* get new ball Y motion, based on where we hit paddle. */
		  /*......................................................*/
		  /* if ball is moving down, it keeps moving down.        */
		  /* if ball is moving up, we need negative of value      */
		  /* from table. Note that this would be faster if the    */
		  /* table hadn't changed to be all positives.            */
		  /*......................................................*/

		 /* if( Ball_Dy == 0 )
		  {
			ballResetDeltaY();
			if( Ball_DeltaIX < 12 )
			{
			    ballFlipDeltaY();
			}
		  }*/
		  if( Ball_Dy >= 0 )
		  {
			ballResetDeltaY();
		  }
		  else
		  {
			ballResetDeltaY();
			ballFlipDeltaY();
		  }

		  /*.....................................................*/
		  /* Since we have hit the paddle, we need to flip the   */
		  /* X motion, and speed the ball up.                    */
		  /* note that speedup() does this every return :)       */
		  /*.....................................................*/
		  /* also note the order: BallDx is > 0, so we speed the */
		  /* ball up before we flip it.                          */
		  /*.....................................................*/


		  speedup();

		  /*.....................................................*/
		  /* limit the speed of the ball.                        */
		  /*.....................................................*/
		  
		  if( Ball_Dx >= 8 )
		  {
				Ball_Fdx = 0;
		  }
		  ballFlipDeltaX(); 
		  
	   }
	   return( 0 );
	}
	return( 0 );
}

/***************************************************************************/
/* speedup(), checks the index just like the angle lookup tables, and      */
/*            decides how much x speed to add                              */
/*.........................................................................*/
/* note: speedup is expecting Ball Delta X to be positive. If Delta X is   */
/* Negative, it will actually slow the ball down.                          */
/*.........................................................................*/

speedup()
{
#asm	
	;---------------------------------------------------------------------
	; load index, make positive offset
	
	lda		_Ball_DeltaIX
	clc				; range is -8 to 31
	adc		#8		; make lowest 'hit' index 0
	tax
	
	;---------------------------------------------------------------------
	; check for bad value. Abort if we have one
	
	bmi		.done
	cmp		#41		; past end of table?
	bpl		.done 
	
	;---------------------------------------------------------------------
	; add fraction to current delta

	clc
	lda		speeduptab,x
	adc		_Ball_Fdx
	sta		_Ball_Fdx
	
	;-------------------------------------------------------------------------
	; add in any carries

	cla
	adc _Ball_Dx 
	sta _Ball_Dx
	
	cla
	adc _Ball_Dx+1 
	sta _Ball_Dx+1
.done
#endasm			  
}	

/***************************************************************************/
/* ball_Reset() - reset ball to initial position.                          */
/*-------------------------------------------------------------------------*/

ball_Reset()
{
    Ball_X         = 124;
	Ball_Fx        = 0;
	Ball_Y         = 120;
	Ball_Fy        = 0;
	
	ball_SetupSprite();
}
/***************************************************************************/
/* ball_Update() - update ball, based on current state.                    */
/*-------------------------------------------------------------------------*/

ball_Update()
{
    if( BallState == 0 )
	{
	   /* state 0 is initial startup. pick direction, start counting. */
	   
	   while( Ball_Dx == 0 )
	   {
		   Ball_Dx      = random( 3 ) -1;
	   }
	   Ball_DeltaIX = random( 32 ) - 16;
	   ballResetDeltaY();
	   
	   BallWaitCount = BALL_DELAY;
	   BallState     = 1;
	   return;
	}
	if( BallState == 1 )
	{
	   /* state 1 is waiting to launch. countdown timer */
	   
	   if( Serving == 0 ) put_string( "VOLLEY 4 SERVE", 9, 0 );
	   if( Serving == 1 ) put_string( " I PINGING II   ", 9, 0 );
	   if( Serving == 2 ) put_string( " II PONGING I   ", 9, 0 );
	   
	   BallWaitCount = BallWaitCount -1;
	   if( BallWaitCount <= 0 )
	   {
	      BallState = 2;
	   }
	   /* if at least 1 sec elapsed, player can serve with 'I' */
	   if( BallWaitCount < BALL_DELAY - 60 )
	   {
	      if( Serving != 0 )
		  {
		      if( joy_I_Pressed( Serving-1 ) )
			  {
			     BallState = 2;
			  }
		  }
	   }
	   return;
	}
	if( BallState == 2 )
	{
		BallSide = ball_Move();
		if(BallSide != 0)
		{
			BallState = 3;
			return;
		}
	   return;
	}
	if( BallState == 3 )
	{
	   /* state 3 is 'point scored'. reset ball, head towards whoever missed */

	   Ball_Dx  = BallSide;
	   Ball_Fdx = 0;
	   Ball_DeltaIX = random( 16 ) -8;
	   ballResetDeltaY();

	   BallState = 4;
	   if( BallSide < 0 )
	   {
			p2_AddPoint();
			Serving = 2;
	   }
	   if( BallSide > 0 )
	   {
	        p1_AddPoint();
			Serving = 1;
	   }
	   return;
	}
	if( BallState == 4 )
	{
	   /* state 4 is reset for new launch. */
	   
	   CompMoveCount     = 1;
	   CompMoveDirection = 0;
	   
	   ball_Reset();

	   BallWaitCount = BALL_DELAY;
	   BallState     = 1;
	}
}

/***************************************************************************/
/* ball_UpdateTrails() - update ball trails.                               */
/*-------------------------------------------------------------------------*/

ball_UpdateTrails()
{
	trailFrame++;
	for( TrailIx = BallSpriteNo + BallTrails -1; TrailIx > BallSpriteNo; TrailIx-- )
	{
	    spr_set( TrailIx-1 );
		TrailX = spr_get_x( );
		TrailY = spr_get_y( );
		
		spr_set( TrailIx );
		spr_x( TrailX );
		spr_y( TrailY );

		/* this flickers the trail. It could use work, probably. */
		if(trailFrame == 3)
		{
			spr_show(TrailIx);
		}
		else
		{
			spr_hide(TrailIx);
		}
	}
	if(trailFrame == 3)
		trailFrame = 0;
}