/***************************************************************************/
/* Paddle2.c - assembler utility functions for managing paddles.           */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 2011 By Michael E. Ward and Andrew 'Arkhan' Darovich		*/
/*				All Rights Reserved.               										*/
/*.........................................................................*/


/***************************************************************************/
/* p1_Set_Up_Dy() - set paddle Delta to -(current motion), for moving up.  */
/*-------------------------------------------------------------------------*/

p1_Set_Up_Dy()
{
#asm
		sec
		cla
		sbc		_Paddle_Cur_Fdy
		sta		_Paddle_1_Fdy
		
		cla	
		sbc		_Paddle_Cur_Dy
		sta		_Paddle_1_Dy
		
		cla
		sbc		_Paddle_Cur_Dy+1
		sta		_Paddle_1_Dy+1
#endasm
}
/***************************************************************************/
/* p1_Set_Down_Dy() - Set paddle delta to current motion, for moving down. */
/*-------------------------------------------------------------------------*/

p1_Set_Down_Dy()
{
#asm
		lda		_Paddle_Cur_Fdy
		sta		_Paddle_1_Fdy
		
		lda		_Paddle_Cur_Dy
		sta		_Paddle_1_Dy
		
		lda		_Paddle_Cur_Dy+1
		sta		_Paddle_1_Dy+1
#endasm
}

/***************************************************************************/
/* p1_Update_Y() - update paddle Vertical position.                        */
/*.........................................................................*/
/* we add current delta to current position for new position.              */
/* current delta comes from current motion, so this setup allows the ball  */
/* to vary its speed by changing current motion.                           */
/*-------------------------------------------------------------------------*/

p1_Update_Y()
{
#asm
		clc
		lda		_Paddle_1_Fdy
		adc		_Paddle_1_Fy
		sta		_Paddle_1_Fy
		
		lda		_Paddle_1_Dy
		adc		_Paddle_1_Y
		sta		_Paddle_1_Y
		
		lda		_Paddle_1_Dy+1
		adc		_Paddle_1_Y+1
		sta		_Paddle_1_Y+1
#endasm
}
/***************************************************************************/
/* p1_Update_X() - update paddle Horizontal position.                      */
/*.........................................................................*/
/* we add current delta to current position for new position.              */
/* current delta comes from current motion, so this setup allows the ball  */
/* to vary its speed by changing current motion.                           */
/*-------------------------------------------------------------------------*/

p1_Update_X()
{
#asm
		clc
		lda		_Paddle_1_Fdx
		adc		_Paddle_1_Fx
		sta		_Paddle_1_Fx
		
		lda		_Paddle_1_Dx
		adc		_Paddle_1_X
		sta		_Paddle_1_X
		
		lda		_Paddle_1_Dx+1
		adc		_Paddle_1_X+1
		sta		_Paddle_1_X+1
#endasm
}
/***************************************************************************/
/* p1_To_Net() - Set paddle delta to motion towards net.                   */
/*-------------------------------------------------------------------------*/

p1_To_Net()
{
#asm
		lda		_Paddle_Cur_In_Fdx
		sta		_Paddle_1_Fdx
		
		lda		_Paddle_Cur_In_Dx
		sta		_Paddle_1_Dx
		
		lda		_Paddle_Cur_In_Dx+1
		sta		_Paddle_1_Dx+1
#endasm
}
/***************************************************************************/
/* p1_From_Net() - Set paddle delta to motion away from net.               */
/*-------------------------------------------------------------------------*/

p1_From_Net()
{
#asm
		sec
		cla
		sbc		_Paddle_Cur_Out_Fdx
		sta		_Paddle_1_Fdx
		
		cla
		sbc		_Paddle_Cur_Out_Dx
		sta		_Paddle_1_Dx
		
		cla
		sbc		_Paddle_Cur_Out_Dx+1
		sta		_Paddle_1_Dx+1
#endasm
}
/***************************************************************************/
/* PLAYER 2                                                                */
/***************************************************************************/

/***************************************************************************/
/* p2_Set_Up_Dy() - set paddle Delta to -(current motion), for moving up.  */
/*-------------------------------------------------------------------------*/

p2_Set_Up_Dy()
{
#asm
		sec
		cla
		sbc		_Paddle_Cur_Fdy
		sta		_Paddle_2_Fdy
		
		cla	
		sbc		_Paddle_Cur_Dy
		sta		_Paddle_2_Dy
		
		cla
		sbc		_Paddle_Cur_Dy+1
		sta		_Paddle_2_Dy+1
#endasm
}
/***************************************************************************/
/* p2_Set_Down_Dy() - Set paddle delta to current motion, for moving down. */
/*-------------------------------------------------------------------------*/

p2_Set_Down_Dy()
{
#asm
		lda		_Paddle_Cur_Fdy
		sta		_Paddle_2_Fdy
		
		lda		_Paddle_Cur_Dy
		sta		_Paddle_2_Dy
		
		lda		_Paddle_Cur_Dy+1
		sta		_Paddle_2_Dy+1
#endasm
}
/***************************************************************************/
/* p2_Update_Y() - update paddle Vertical position.                        */
/*.........................................................................*/
/* we add current delta to current position for new position.              */
/* current delta comes from current motion, so this setup allows the ball  */
/* to vary its speed by changing current motion.                           */
/*-------------------------------------------------------------------------*/

p2_Update_Y()
{
#asm
		clc
		lda		_Paddle_2_Fdy
		adc		_Paddle_2_Fy
		sta		_Paddle_2_Fy
		
		lda		_Paddle_2_Dy
		adc		_Paddle_2_Y
		sta		_Paddle_2_Y
		
		lda		_Paddle_2_Dy+1
		adc		_Paddle_2_Y+1
		sta		_Paddle_2_Y+1
#endasm
}
/***************************************************************************/
/* p2_Update_X() - update paddle Horizontal position.                      */
/*.........................................................................*/
/* we add current delta to current position for new position.              */
/* current delta comes from current motion, so this setup allows the ball  */
/* to vary its speed by changing current motion.                           */
/*-------------------------------------------------------------------------*/

p2_Update_X()
{
#asm
		clc
		lda		_Paddle_2_Fdx
		adc		_Paddle_2_Fx
		sta		_Paddle_2_Fx
		
		lda		_Paddle_2_Dx
		adc		_Paddle_2_X
		sta		_Paddle_2_X
		
		lda		_Paddle_2_Dx+1
		adc		_Paddle_2_X+1
		sta		_Paddle_2_X+1
#endasm
}
/***************************************************************************/
/* p2_To_Net() - Set paddle delta to motion towards net.                   */
/*-------------------------------------------------------------------------*/

p2_To_Net()
{
#asm
		sec
		cla
		sbc		_Paddle_Cur_In_Fdx
		sta		_Paddle_2_Fdx
		
		cla
		sbc		_Paddle_Cur_In_Dx
		sta		_Paddle_2_Dx
		
		cla
		sbc		_Paddle_Cur_In_Dx+1
		sta		_Paddle_2_Dx+1
#endasm
}
/***************************************************************************/
/* p2_From_Net() - Set paddle delta to motion away from net.               */
/*-------------------------------------------------------------------------*/

p2_From_Net()
{
#asm
		lda		_Paddle_Cur_Out_Fdx
		sta		_Paddle_2_Fdx
		
		lda		_Paddle_Cur_Out_Dx
		sta		_Paddle_2_Dx
		
		lda		_Paddle_Cur_Out_Dx+1
		sta		_Paddle_2_Dx+1
#endasm
}
