/***************************************************************************/
/* Rock.c - functions for managing rocks.                                  */
/*-------------------------------------------------------------------------*/
/* parameter block variables for spawn operation.                          */

int  spawnIndex;
int  spawnType;
int  spawnX;
int  spawnY;
int  spawnH;

char rocksActive;
int  one;
int two;
int three;

/***************************************************************************/
/* RockLoad() - load rock resources into memory.                           */
/*-------------------------------------------------------------------------*/

rockLoad()
{
    load_sprites( RockAddress,        RockSprite,  4 );
	load_palette( RockPaletteNo + 16, RockPalette, 1 );
}

/***************************************************************************/
/* rockSetup() - initialize rock, including it's sprite.                   */
/*-------------------------------------------------------------------------*/

rockSetup()
{
   int  i;
   
   for( i = 0; i < MAX_ROCKS; i ++ )
   {
      RockActive[ i ] = 0;
      spr_hide( FIRST_ROCK + i  );
   }
}
/***************************************************************************/
/* rockSpawn() - initialize rock, including it's sprite.                   */
/*-------------------------------------------------------------------------*/

rockSpawn( rock, size )
int        rock, size;
{
    spawnIndex = rock;

	if( size == 64 )
	{
	  spawnType = ROCK_64X64L;
   	  spawnX = 128;
	  spawnY = 120;
	  spawnH = 6;
	  
      SpawnRock();
	  SetupRockSprite();

      /*.......................................................................*/
	  /* select slave sprite.                                                  */
      /*.......................................................................*/

	  spawnX     = spawnX + 32;
	  spawnIndex = spawnIndex + 4;
      spawnType  = spawnType + 1;
	}
	if( size == 32 )
	{
	  spawnType = ROCK_32X32;
	  spawnX = 64;
	  spawnY = 64;
	  spawnH = 2;
	}
	if( size == 16 )
	{
	  spawnType = ROCK_16X16;

	  spawnX = 16;
	  spawnY = 16;
	  spawnH = 4;
	}
	if( size == 8 )
	{
	  spawnType = ROCK_8X8;
	  
	  spawnX = 200;
	  spawnY = 200;
	  spawnH = 6;
	}
    SpawnRock();
    SetupRockSprite();
}
/***************************************************************************/
/* DisplayRocks() - display ALL rock sprites.                              */
/*-------------------------------------------------------------------------*/
/* this could use some optimizing. Registers instead of variables...       */

DisplayRocks()
{
   #asm
   
    clx
   ;----------------------------------------------------------------
   ; while rock_no < MAX_ROCKS
   
rDLoop

   ;----------------------------------------------------------------
   ; see if rock is active.
   
   lda		_RockActive, x
   beq		.noUpdate
   
   ;----------------------------------------------------------------
   ; update rock display

   phx						; rDisplay trashes All registers.
   call _rDisplay
   plx						; restore saved rock number
   
.noUpdate

   ;----------------------------------------------------------------
   ; calculate next increment, add to index

   lda		_RockType,x		; get type of rock
   tay
   
   txa						; get rock_no
   clc
   adc		_rockSprites,y  ; add # sprites to skip

   ;----------------------------------------------------------------
   ; add increment to index
 
   cmp		#_MAX_ROCKS		; out of rocks ?
   bcs		rDLoopOut
   
   tax						; save as next index
   jmp		rDLoop
   
rDLoopOut
   #endasm
}
/***************************************************************************/
/* UpdateRocks() - update ALL rock sprites.                                */
/*-------------------------------------------------------------------------*/

UpdateRocks()
{
   #asm
   stz		_rocksActive
   clx						; x is rock number

   ;----------------------------------------------------------------
   ; while rock_no < MAX_ROCKS
   
rULoop

   ;----------------------------------------------------------------
   ; calculate next increment

   ldy		_RockType,x		; get type of rock
   
   ;----------------------------------------------------------------
   ; see if rock is active.
   
   lda		_RockActive, x
   beq		noUpdate

   ;----------------------------------------------------------------
   ; rock is active. see if it's time to move it.
   
    inc			_rocksActive
	
    lda			_RockDelay,x			; get delay timer
	dec			a
    sta			_RockDelay,x			; save updated timer
	bne			noUpdate				; still have time left

    lda			_rockDelay,y			; times up - reset timer
    sta			_RockDelay,x
	
   ;----------------------------------------------------------------
   ; update rock position, wrap coordinates

    phy									; save rock type
	call	_rockAddDeltas				; move rock
    call    _rockScreenWrap				; wrap coord if needed
	ply									; restore rock type
	
noUpdate

   ;----------------------------------------------------------------
   ; add increment to index

   txa									; copy rock number
   clc
   adc		_rockSprites,y				; bump by # sprites
   cmp		#_MAX_ROCKS					; all done ?
   bcs		rULoopOut					; yep, exit

   tax									; save new rock number
   jmp  rULoop							; and do next rock

rULoopOut
   #endasm
}

/***************************************************************************/
/* rockAddDeltas() - add DeltaX and DeltaY to shot coordinates.            */
/*-------------------------------------------------------------------------*/
/* this is a 3 byte add, with fractions in the lsb.                        */
/* we now expect X to hold the rock number.                                */
/*.........................................................................*/

rockAddDeltas()
{
#asm
	;-----------------------------------------------------------------
	; update ScreenX
	
	clc
	lda		_RockDeltaXf,x
	adc		_RockScreenXf,x
	sta		_RockScreenXf,x
	
	lda		_RockDeltaXl,x
	adc		_RockScreenXl,x
	sta		_RockScreenXl,x
	
	lda		_RockDeltaXh,x	
	adc		_RockScreenXh,x
	sta		_RockScreenXh,x
	
	;-----------------------------------------------------------------
	; update ScreenY
	
	clc
	lda		_RockDeltaYf,x
	adc		_RockScreenYf,x
	sta		_RockScreenYf,x
	
	lda		_RockDeltaYl,x
	adc		_RockScreenYl,x
	sta		_RockScreenYl,x
	
	lda		_RockDeltaYh,x
	adc		_RockScreenYh,x
	sta		_RockScreenYh,x
	
#endasm
}

/**********************************************************************************/
/* rockScreenWrap() - wrap sprite coordinates to screen.                          */
/*................................................................................*/
/* rock number to wrap is in X.                                                   */
/*................................................................................*/

int  wrapX;
int  wrapY;

rockScreenWrap()
{

	/*..................................................................*/
	/* wrap if off screen edges. RockScreenX and RockScreenY are center */
	/* of sprite.          												*/
	/*..................................................................*/
	
    #asm
	;--------------------------------------------------------------------
    ; get rock type index (in y)

    lda		_RockType,x		; get type of rock
	asl		a				; 2 bytes for an int
    tay
	
	;.................................................................
	; did it go off left edge of screen ? (ie, is < 0 )
	;.................................................................
	; we WANT RockScreenX < LeftEdge; but can only check <=.
	; so, we check LeftEdge <= RockScreenX, and reverse the outcome.
	
	sec
	lda		_RockScreenXl,x         ; RockScreenX
	sbc		_scrLeft,y				; - LeftEdge
	lda		_RockScreenXh,x
	sbc		_scrLeft+1,y
	
    bmi		.wrap1
    jmp		.notLeft
	 
	;..................................................................
    ;  if off left, wrap to right edge
.wrap1

    lda		_scrRight,y
    sta		_RockScreenXl, x
    lda		_scrRight+1,y
	sta		_RockScreenXh,x
    jmp     .XWrapped	

	;-----------------------------------------------------------------
	; calculate where right edge of screen would be.
	
.notLeft

	;.................................................................
	; did it go off right edge of screen ? (ie, is > SCREEN_W  )

	sec
	lda		_RockScreenXl,x
	sbc		_scrRight,y
    lda		_RockScreenXh,x	
	sbc		_scrRight+1,y
	bmi     .notRight		; RockScreenX < wrapX ?
	
	;..................................................................
    ;  if off right, wrap to left edge

    lda		_scrLeft,y
    sta		_RockScreenXl, x
    lda		_scrLeft+1,y
	sta		_RockScreenXh,x

	;*****************************************************************
    ; process top/bottom wrap
	
.notRight
.XWrapped

	;..................................................................
	; did it go off top edge of screen ? (ie, is < 0 )

	sec
	lda		_RockScreenYl,x
	sbc		_scrTop,y
    lda		_RockScreenYh,x	
	sbc		_scrTop+1,y
	bpl     .notTop		; RockScreenX < wrapX ?
	
	;..................................................................
    ;  if off top, wrap to bottom edge

.wrapTop
    lda		_scrBottom,y
    sta		_RockScreenYl, x
    lda		_scrBottom+1,y
	sta		_RockScreenYh,x
    jmp     .YWrapped	

.notTop
    ;..................................................................
	; did it go off bottom edge of screen ?

	sec
	lda		_RockScreenYl,x
	sbc		_scrBottom,y
    lda		_RockScreenYh,x	
	sbc		_scrBottom+1,y
	bmi     .notBottom		; RockScreenX < wrapX ?
	
	;----------------------------------------------------------------
    ;  if off bottom, wrap to top edge

    lda		_scrTop,y
    sta		_RockScreenYl, x
    lda		_scrTop+1,y
	sta		_RockScreenYh,x
	
.notBottom
.YWrapped

    #endasm
}
/**********************************************************************************/
/* rDisplay() - display sprite.                                                   */
/*................................................................................*/
/* we expect rock number to display in X.                                         */
/* We trash all the registers, so save anyone you might need later.               */

rDisplay()
{
#asm 

  ;-------------------------------------------------------------------
  ; just because, this should be how to do a (HuC compatible) spr_set
  ; in assembly.
  ;-------------------------------------------------------------------
  ; HuC low-level 'functions' use A:X for params, with A being high byte
  
  phx						; save rock number
  
  txa
  clc
  adc	#_FIRST_ROCK
  tax						; sprite number goes in X. we can ignore A
  call  _spr_set  			; call HuC function. has to be call, to make
                            ; sure it's mapped in.

  ;-------------------------------------------------------------------
  ; how about a spr_x ???

  plx							; restore rock number
  lda		_RockType,x			; load rock type (for offset)
  asl		a					; 2 bytes per int
  tay							; save index

  phx							; save, cause it gets trashed
  phy							; this apparently gets trashed, too
  
  ;...................................................................
  ; subtract low byte

  sec
  lda		_RockScreenXl,x
  sbc		_RockSpriteOffset,y
  pha							; save for later

  lda		_RockScreenXh,x
  sbc		_RockSpriteOffset+1,y

  plx							; restore low byte
  call		_spr_x  

  ;-------------------------------------------------------------------
  ; and a spr_y ....I don't know if Y changes or not, so I re-load it
  ; X definately got changed :-)

  ply							; restore old indexes
  plx  
  
  ;...................................................................
  ; subtract low byte

  sec
  lda		_RockScreenYl,x
  sbc		_RockSpriteOffset,y
  pha							; save for later

  lda		_RockScreenYh,x
  sbc		_RockSpriteOffset+1,y

  plx							; restore low byte
  call		_spr_y
#endasm
}
