/***************************************************************************/
/* Ship.c - functions for managing ships.                                  */
/*-------------------------------------------------------------------------*/

/***************************************************************************/
/* shipLoad() - load ship resources into memory.                           */
/*-------------------------------------------------------------------------*/

shipLoad()
{
    load_sprites( ShipAddress,        ShipSprite,   3 );
	
	/*..................................................................*/
	/* sprites have a seperate set of palettes, from 16 to 32.          */
	/* to use the ship palette for the score, we have to load it        */
	/* into the background palete area (0-15).                          */
	/*..................................................................*/
	
	load_palette( ShipPaletteNo     , Ship1Palette, 1 );
	load_palette( ShipPaletteNo + 16, Ship1Palette, 1 );
	
	load_palette( ShipPaletteNo +  1, Ship2Palette, 1 );
	load_palette( ShipPaletteNo + 17, Ship2Palette, 1 );
	
	load_palette( ShipPaletteNo +  2, Ship3Palette, 1 );
	load_palette( ShipPaletteNo + 18, Ship3Palette, 1 );

	load_palette( ShipPaletteNo +  3, Ship4Palette, 1 );
	load_palette( ShipPaletteNo + 19, Ship4Palette, 1 );
}
/***************************************************************************/
/* shipPositionIcons() - initialize ship, including it's sprite, as icon   */
/*-------------------------------------------------------------------------*/

shipPositionIcons()
{
   int  i;
   
   for( i = 0; i < MAX_SHIPS; i ++ )
   {   
  	    spr_set(     ShipSpriteNo + i );
	    spr_x( (  6 + 4 * i) * 8 );
	    spr_y( (  5        ) * 8 );
        spr_pattern( ShipAddress   + 128    );
	    spr_pal(     ShipPaletteNo + 16 + i );
	    spr_hide(    ShipSpriteNo  +      i );
	    spr_pri( 1 );
		
		ShipState[i] = 0;
    }	

}

/***************************************************************************/
/* shipSetup() - initialize ship, including it's sprite.                   */
/*-------------------------------------------------------------------------*/

shipSetup()
{
   int  i;
   
   for( i = 0; i < MAX_SHIPS; i ++ )
   {   
		shipReset( i );
		ShipLeft[ i ] = INITIAL_SHIPS - 1;
		ShotsFired[i] = 0;
   }	

}
/***************************************************************************/
/* shipReset() - initialize ship, including it's sprite.                   */
/*-------------------------------------------------------------------------*/

shipReset( shipNo )
int        shipNo;
{
	ShipScreenXh[shipNo]  = shipInitXh[shipNo];
	ShipScreenXl[shipNo]  = shipInitXl[shipNo];	
	ShipScreenXf[shipNo]  = 0;

	ShipScreenYh[shipNo]  = shipInitYh[shipNo];
	ShipScreenYl[shipNo]  = shipInitYl[shipNo];    
	ShipScreenYf[shipNo]  = 0;

	ShipDeltaXh[shipNo]   = 0;
	ShipDeltaXl[shipNo]   = 0;
	ShipDeltaXf[shipNo]   = 0;

	ShipDeltaYh[shipNo]   = 0;
	ShipDeltaYl[shipNo]   = 0;
	ShipDeltaYf[shipNo]   = 0;

	ShipHeading[shipNo]  		= shipInitHeading[shipNo];
	ShipCurrentAddress[shipNo]  = ShipAddress;

	ShipThrusting[shipNo]        = 0;

	/*............................................................*/
	/* initialize sprite information.                             */
	/*............................................................*/

	spr_set(     ShipSpriteNo + shipNo );
	spr_x(       ShipScreenXh[shipNo] * 256 + ShipScreenXl[shipNo] - 7 );
	spr_y(       ShipScreenYh[shipNo] * 256 + ShipScreenYl[shipNo] - 7 );
    spr_pattern( ShipCurrentAddress[ shipNo ] );
	spr_pal(     ShipPaletteNo + 16 + shipNo  );
	spr_hide(    ShipSpriteNo + shipNo );
	spr_pri( 1 );
	
}
/***************************************************************************/
/* shipActivate() - activate ship(s).                                      */
/*-------------------------------------------------------------------------*/

shipActivate()
{
   int  i;
   
   for( i = 0; i < MAX_SHIPS; i ++ )
   {  
      if( ShipState[i] & SHIP_ACTIVE )
	  {
		ShipState[ i ] |= SHIP_ALIVE;
	  }
   }	

}
/***************************************************************************/
/* shipDisplay() - display ship sprite.                                    */
/*-------------------------------------------------------------------------*/

shipDisplay()
{
    /*
    SHIP_TRACE_ON
	*/
    for( ship_no = 0; ship_no < MAX_SHIPS; ship_no ++ )
	{
	    if( ShipState[ ship_no ] & SHIP_ACTIVE )
		{
		    /*............................................................*/
			/* use explosion graphics if ship is dying.                   */
			/*............................................................*/
			
			if( ShipState[ship_no] & SHIP_DYING )
			{
				ShipCurrentAddress[ship_no] = ShipAddress + 0x400 + ShipFrame[ship_no] * 64;
			}
			spr_set(     ShipSpriteNo + ship_no );
			spr_x(       ShipScreenXh[ship_no] * 256 + ShipScreenXl[ship_no] - 7 );
			spr_y(       ShipScreenYh[ship_no] * 256 + ShipScreenYl[ship_no] - 7 );
			spr_pattern( ShipCurrentAddress[ship_no] );

		    /*............................................................*/
			/* hide ship if dead.                                         */
		    /*............................................................*/

			if( ShipState[ship_no] & SHIP_DEAD )
			{
				spr_hide(    ShipSpriteNo + ship_no  );
			}
			else
			{
				spr_show(    ShipSpriteNo + ship_no  );
			}
		}
	}
	/*
	SHIP_TRACE_OFF
	*/
}

/***************************************************************************/
/* shipProcessMoves() - process moves for all ships.                       */
/*-------------------------------------------------------------------------*/

shipProcessMoves()
{
  /*
  SHIP_TRACE_ON
  */
  
  for( ship_no = 0; ship_no < MAX_SHIPS; ship_no++ )
  {
	  /*...................................................................*/
	  /* only handle activated ships                                       */
	  /*...................................................................*/
	  
	  if( ShipState[ ship_no ] & SHIP_ACTIVE )
	  {
		  /*...................................................................*/
		  /* if ship is dead, wait out delay and check for any left.           */
		  /*...................................................................*/
		  
		  if( ShipState[ ship_no ] & SHIP_DEAD )
		  {
			  shipReSpawn();
			  continue;
		  }
		  /*...................................................................*/
		  /* if ship is dying, step explosion.                                 */
		  /*...................................................................*/
		  
		  if( ShipState[ ship_no ] & SHIP_DYING )
		  {
			  shipExplode();
			  continue;
		  }

		  /*...................................................................*/
		  /* if ship is alive, handle keys and update position.                */
		  /*...................................................................*/
		  
		  if( ShipState[ ship_no ] & SHIP_ALIVE )
		  {
			  /*...................................................................*/
			  /* handle user input.                                                */
			  /*...................................................................*/
			  
			  shipProcessKeys();

			  /*...................................................................*/
			  /* update ship position.                                             */
			  /*...................................................................*/
			  
			  shipUpdate();
		  }
	  }
    }
	/*
	SHIP_TRACE_OFF
	*/
}
/***************************************************************************/
/* shipExplode() - display ship exploding sprite.                          */
/*-------------------------------------------------------------------------*/

shipExplode()
{
    if( ShipFrame[ship_no] > 8 ) 
	{
	   ShipState[      ship_no ] = (ShipState[ship_no] & 0xF0) | SHIP_DEAD;
	   ShipFrameDelay[ ship_no ] = 0;
	   ShipFrame[      ship_no ] = 0;
	   return;
	}

    ShipFrameDelay[ ship_no ]++;
	if( ShipFrameDelay[ship_no] > SHIP_EXPLODE_DELAY )
	{
	   ShipFrame[ship_no] ++;
	   ShipFrameDelay[ship_no] = 0;
	}
}
/***************************************************************************/
/* shipReSpawn() - wait out death delay, then respawn ship, if any left.   */
/*-------------------------------------------------------------------------*/

shipReSpawn()
{
	if( ShipLeft[ship_no ] )
	{
		ShipFrameDelay[ ship_no ]++;
		if( ShipFrameDelay[ ship_no ] > SHIP_SPAWN_DELAY )
		{
		   ShipState[ ship_no ] = ( ShipState[ship_no] & 0xF0 ) | SHIP_ALIVE;
		   shipReset( ship_no );

		   ShipFrameDelay[ship_no] = 0;
		   ShipLeft[ship_no]--;
		}
	}
}
/***************************************************************************/
/* shipUpdate() - update ship sprite.                                      */
/*-------------------------------------------------------------------------*/

shipUpdate()
{
    /*..................................................................*/
	/* calculate base graphics address.                                 */
	/*..................................................................*/
	
    ShipCurrentAddress[ship_no] = ShipAddress + ShipHeading[ship_no] * 64;

    /*..................................................................*/
	/* handle thrusters.                                                */
    /*..................................................................*/
	
	if( ShipThrusting[ship_no] )
	{
	   /*.........................................................*/
	   /* add in new velocity components.                         */
	   /*.........................................................*/

	   #asm
	   ldy		_ship_no
	   ldx		_ShipHeading,y
	   
	   clc
	   lda		_xthrust_f,x
	   adc		_ShipDeltaXf,y
	   sta		_ShipDeltaXf,y
	   lda		_xthrust_l,x
	   adc		_ShipDeltaXl,y
       sta		_ShipDeltaXl,y
	   lda		_xthrust_h,x
	   adc		_ShipDeltaXh,y
       sta		_ShipDeltaXh,y

	   clc
	   lda		_ythrust_f,x
	   adc		_ShipDeltaYf,y
	   sta		_ShipDeltaYf,y
	   lda		_ythrust_l,x
	   adc		_ShipDeltaYl,y
       sta		_ShipDeltaYl,y
	   lda		_ythrust_h,x
	   adc		_ShipDeltaYh,y
       sta		_ShipDeltaYh,y
	   #endasm
	   
	   /*.........................................................*/
	   /* limit velocity.                                         */
	   /*.........................................................*/
	   
	   shipCapDeltaX();
       shipCapDeltaY();	   

	   /*.........................................................*/
	   /* change to graphics row with thrusters on.               */
	   /*.........................................................*/

       #asm
	   ldy		_ship_no
       clc
       lda		#02
       adc      _ShipCurrentAddress+1,y
       sta		_ShipCurrentAddress+1,y	   
	   #endasm
	}
	
    /*..................................................................*/
	/* update ship position.                                            */
    /*..................................................................*/
	
	shipAddDeltas();

    /*..................................................................*/
	/* bounce off screen edges.                                         */
    /*..................................................................*/

	shipScreenCheck();
	
}
/***************************************************************************/
/* shipAddDeltas() - add DeltaX and DeltaY to ship coordinates.            */
/*-------------------------------------------------------------------------*/
/* this is a 3 byte add, with fractions in the lsb.                        */
/*.........................................................................*/

shipAddDeltas()
{
#asm

	ldx		_ship_no
	
	;-----------------------------------------------------------------
	; update ScreenX
	
	clc
	lda		_ShipDeltaXf,x
	adc		_ShipScreenXf,x
	sta		_ShipScreenXf,x
	
	lda		_ShipDeltaXl,x
	adc		_ShipScreenXl,x
	sta		_ShipScreenXl,x
 
    lda		_ShipDeltaXh,x
	adc		_ShipScreenXh,x
	sta		_ShipScreenXh,x
	
	;-----------------------------------------------------------------
	; update ScreenY
	
	clc
	lda		_ShipDeltaYf,x
	adc		_ShipScreenYf,x
	sta		_ShipScreenYf,x
	
	lda		_ShipDeltaYl,x
	adc		_ShipScreenYl,x
	sta		_ShipScreenYl,x
	
	lda		_ShipDeltaYh,x
	adc		_ShipScreenYh,x
	sta		_ShipScreenYh,x
	
#endasm
}
/***************************************************************************/
/* shipFlipDeltaX() -  complement ships DeltaX.                            */
/*.........................................................................*/

shipFlipDeltaX()
{
#asm
    phx
	ldx		_ship_no
	
	sec
	cla
	sbc		_ShipDeltaXf,x
	sta		_ShipDeltaXf,x
	
	cla
	sbc		_ShipDeltaXl,x
	sta		_ShipDeltaXl,x

	cla
	sbc		_ShipDeltaXh,x
	sta		_ShipDeltaXh,x

	plx
#endasm
}
/***************************************************************************/
/* shipFlipDeltaY() -  complement ships DeltaY.                            */
/*.........................................................................*/

shipFlipDeltaY()
{
#asm
    phx
	ldx		_ship_no

	sec
	cla
	sbc		_ShipDeltaYf,x
	sta		_ShipDeltaYf,x
	
	cla
	sbc		_ShipDeltaYl,x
	sta		_ShipDeltaYl,x

	cla
	sbc		_ShipDeltaYh,x
	sta		_ShipDeltaYh,x
	
	plx
#endasm
}

/***************************************************************************/
/* shipRecoil() -  apply shot recoil, in opposite direction of heading.    */
/*.........................................................................*/

shipRecoil()
{
#asm
    phx							; preserve X. We Might Need It.
    phy
	
	ldy		_ship_no
	lda		_ShipHeading,y		; recoil is in opposite direction
	clc
	adc		#4
	and		#$07
	sta		_ship_heading

	tax
	
	;-------------------------------------------------------------------
    ; check to see if cos is + or minus

    stz		_trig+3			; assume positive
	lda		_cosh,x
	bpl		.skip1
	dec		_trig+3

	;-------------------------------------------------------------------
	; calculate cos[h] / 16; save in trig for subtract

.skip1
	lsr     _trig+3			; get sign
	ror     a				; shift high byte
	sta		_trig+2
	
	lda		_cosl,x
	ror		a
	sta		_trig+1
	
	lda		_cosf,x
	ror		a
	sta		_trig

	;-------------------------------------------------------------------
    ; that was 1. now do 3 more. right after I make sure it worked :-)
	
    ldy		#3
.loop1
    lsr     _trig+3
	ror     _trig+2
	ror     _trig+1
	ror     _trig

	dey
	beq		.skip2
	jmp		.loop1
.skip2

	;-------------------------------------------------------------------
	; add to the ships deltaX
	
	clc
	ldy		_ship_no
	lda		_trig
	adc		_ShipDeltaXf,y
	sta		_ShipDeltaXf,y
	lda		_trig+1
	adc		_ShipDeltaXl,y
	sta		_ShipDeltaXl,y
	lda		_trig+2
	adc		_ShipDeltaXh,y
	sta		_ShipDeltaXh,y
	
	;-------------------------------------------------------------------
    ; check to see if sin is + or minus

    stz		_trig+3			; assume positive
	lda		_sinh,x
	bpl		.skip3
	dec		_trig+3

	;-------------------------------------------------------------------
	; calculate sin[h] / 16; save in trig for subtract

.skip3
	lsr     _trig+3			; get sign
	ror     a				; shift high byte
	sta		_trig+2
	
	lda		_sinl,x
	ror		a
	sta		_trig+1
	
	lda		_sinf,x
	ror		a
	sta		_trig

	;-------------------------------------------------------------------
    ; that was 1. now do 3 more. right after I make sure it worked :-)
	
    ldy		#3
.loop2
    lsr     _trig+3
	ror     _trig+2
	ror     _trig+1
	ror     _trig
	
	dey
	beq		.skip4
	jmp		.loop2

.skip4

	;-------------------------------------------------------------------
	; add to the ships deltaY
	
	clc
	ldy		_ship_no
	lda		_trig
	adc		_ShipDeltaYf,y
	sta		_ShipDeltaYf,y
	lda		_trig+1
	adc		_ShipDeltaYl,y
	sta		_ShipDeltaYl,y
	lda		_trig+2
	adc		_ShipDeltaYh,y
	sta		_ShipDeltaYh,y

	ply
	plx
#endasm
}
/***************************************************************************/
/* shipScreenCheck() - check for ship bouncing off screen edges.           */
/*-------------------------------------------------------------------------*/
/* throughout this routine, Y holds the ship number to check.              */
/*.........................................................................*/

shipScreenCheck()
{
    #asm

    ldy	    _ship_no

    ;-------------------------------------------------------------------
    ; horizontal bounce checks.
    ;-------------------------------------------------------------------
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; did it go off left edge of screen ? (ie, is < 0 )
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	sec
	lda		_ShipScreenXl,y         ;  ShipScreenX
	sbc     #7                      ; - ship width
	lda		_ShipScreenXh,y
	sbc		#0
	
    bmi		.off_left				; if ShipScreenX -7 < 0
    jmp		.checkRight				; it's not. Check right edge bounce

	;....................................................................
	; ship went off left side of screen. put it back at left edge....
	;....................................................................

.off_left
	
	lda		#7
	sta		_ShipScreenXl,y
	lda		#0
	sta		_ShipScreenXh,y
	sta		_ShipScreenXf,y

	;....................................................................
	; and flip it's motion.
	;....................................................................
	
	call	_shipFlipDeltaX
	jmp		.checkY					; no need to check right edge. Skip to Y check

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; did it go off right edge of screen ? (ie, is > width )
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
.checkRight:

	sec
	lda		_ShipScreenXl,y         ;  ShipScreenX
	sbc     #LOW(_SCREEN_W -7)      ; - screen width
	lda		_ShipScreenXh,y
	sbc		#HIGH( _SCREEN_W -7)
	
    bpl		.off_right				; ShipScreenX >= Width - 7
    jmp		.checkY					; it's not, skip to Y check
	
	;....................................................................
	; ship went off right side of screen. put it back at right edge....
	;....................................................................

.off_right

	lda		#LOW( _SCREEN_W -7 )
	sta		_ShipScreenXl,y
	lda		#HIGH(_SCREEN_W -7 ) 
	sta		_ShipScreenXh,y
	lda		#0
	sta		_ShipScreenXf,y
	
	;....................................................................
	; and flip it's motion.
	;....................................................................

	call	_shipFlipDeltaX
	
    ;-------------------------------------------------------------------
    ; vertical bounce checks.
    ;-------------------------------------------------------------------

.checkY
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; did it go off top edge of screen ? (ie, is < 0 )
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	sec
	lda		_ShipScreenYl,y         ;  ShipScreenX
	sbc     #7                      ; - ship width
	lda		_ShipScreenYh,y
	sbc		#0
	
    bmi		.off_top
    jmp		.checkBottom

	;....................................................................
	; ship went off top of screen. put back at top....
	;....................................................................
	
.off_top
	
	lda		#7
	sta		_ShipScreenYl,y
	lda		#0
	sta		_ShipScreenYh,y
	sta		_ShipScreenYf,y
	
	;....................................................................
	; and flip it's motion.
	;....................................................................
	
	call	_shipFlipDeltaY
	jmp		.allDone				; no need to check bottom.
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; did it go off bottom edge of screen ? (ie, is > height )
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.checkBottom:

	sec
	lda		_ShipScreenYl,y         ;  ShipScreenX
	sbc     #LOW(_SCREEN_H -7)        ; - screen width
	lda		_ShipScreenYh,y
	sbc		#HIGH( _SCREEN_H -7)
	
    bpl		.off_bottom
    jmp		.allDone

	;.................................................................
	; ship went off bottom of screen. put it back ant bottom....
    ;...................................................................
	
.off_bottom
	
	lda		#LOW( _SCREEN_H -7 )
	sta		_ShipScreenYl,y
	lda		#HIGH(_SCREEN_H -7) 
	sta		_ShipScreenYh,y
	lda		#0
	sta		_ShipScreenYf,y
	
    ;...................................................................
	; and flip it's motion.
    ;...................................................................
	
	call	_shipFlipDeltaY
	
.allDone
    #endasm
}
/***************************************************************************/
/* shipCapDeltaX() - cap ship delta X to max/min.                          */
/*-------------------------------------------------------------------------*/
/* throughout this routine, Y holds the ship number to check.              */
/*.........................................................................*/

shipCapDeltaX()
{
	   #asm
	   ldy		_ship_no
	   
	   ;-----------------------------------------------------------
	   ; if (DeltaX > 1024 ) => DeltaX - 1024 > 0
	   
	   sec
	   lda		_ShipDeltaXf,y
	   sbc		#0
	   lda		_ShipDeltaXl,y
	   sbc		#LOW( _SHIP_MAX_VELOCITY )
	   lda		_ShipDeltaXh,y
	   sbc		#HIGH( _SHIP_MAX_VELOCITY )

       bmi		.x_not_hi				; if < 0, it's okay. 

	   ;-----------------------------------------------------------
	   ; DeltaX too large, cap to 1024
	   
	   lda		#LOW( _SHIP_MAX_VELOCITY )        ; otherwise, cap it
	   sta		_ShipDeltaXl,y
	   lda		#HIGH( _SHIP_MAX_VELOCITY )
	   sta		_ShipDeltaXh,y

	   bra		.xDone

	   
	   ;-----------------------------------------------------------
	   ; if (DeltaX < -1024 ) => DeltaX + 1024 < 0
	   
.x_not_hi

	   sec
	   lda		_ShipDeltaXf,y
       sbc		#0	   
	   lda		_ShipDeltaXl,y
	   sbc		#LOW( _SHIP_MIN_VELOCITY )
       lda		_ShipDeltaXh,y
	   sbc		#HIGH( _SHIP_MIN_VELOCITY )
       bpl		.xDone				; if < 0, it's okay. 

	   ;-----------------------------------------------------------
	   ; DeltaX too small, cap to -1024
	   
	   lda		#LOW( _SHIP_MIN_VELOCITY )        ; otherwise, cap it
	   sta		_ShipDeltaXl,y
	   lda		#HIGH( _SHIP_MIN_VELOCITY )
	   sta		_ShipDeltaXh,y
	   
.xDone
	#endasm
}
/***************************************************************************/
/* shipCapDeltaY() - cap ship delta Y to max/min.                          */
/*-------------------------------------------------------------------------*/
/* throughout this routine, Y holds the ship number to check.              */
/*.........................................................................*/

shipCapDeltaY()
{
	   #asm
	   ldy		_ship_no
	   
	   ;-----------------------------------------------------------
	   ; if (DeltaY > 1024 ) => DeltaY - 1024 > 0
	   
	   sec
	   lda		_ShipDeltaYf,y
	   sbc		#0
	   lda		_ShipDeltaYl,y
	   sbc		#LOW( _SHIP_MAX_VELOCITY )
	   lda		_ShipDeltaYh,y
	   sbc		#HIGH( _SHIP_MAX_VELOCITY )

       bmi		.y_not_hi				; if < 0, it's okay. 

	   ;-----------------------------------------------------------
	   ; DeltaY too large, cap to 1024
	   
	   lda		#LOW( _SHIP_MAX_VELOCITY )        ; otherwise, cap it
	   sta		_ShipDeltaYl,y
	   lda		#HIGH( _SHIP_MAX_VELOCITY )
	   sta		_ShipDeltaYh,y

	   bra		.yDone

	   
	   ;-----------------------------------------------------------
	   ; if (DeltaY < -1024 ) => DeltaY + 1024 < 0
	   
.y_not_hi

	   sec
	   lda		_ShipDeltaYf,y
       sbc		#0	   
	   lda		_ShipDeltaYl,y
	   sbc		#LOW( _SHIP_MIN_VELOCITY )
       lda		_ShipDeltaYh,y
	   sbc		#HIGH( _SHIP_MIN_VELOCITY )
       bpl		.yDone				; if < 0, it's okay. 

	   ;-----------------------------------------------------------
	   ; DeltaY too small, cap to -1024
	   
	   lda		#LOW( _SHIP_MIN_VELOCITY )        ; otherwise, cap it
	   sta		_ShipDeltaYl,y
	   lda		#HIGH( _SHIP_MIN_VELOCITY )
	   sta		_ShipDeltaYh,y
	   
.yDone
	#endasm
}